@@ -30,11 +30,10 @@
 #include "eel-lib-self-check-functions.h"
 #include "eel-enumeration.h"
 #include "eel-glib-extensions.h"
-#include "eel-string-list.h"
 #include "eel-string.h"
 #include <gconf/gconf-client.h>
 #include <gconf/gconf.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 
 /* An enumeration used for updating auto-storage variables in a type-specific way. 
  * FIXME: there is another enumeration like this in eel-global-preferences.c,
@@ -44,7 +43,8 @@ typedef enum {
 	PREFERENCE_BOOLEAN = 1,
 	PREFERENCE_INTEGER,
 	PREFERENCE_STRING,
-	PREFERENCE_STRING_LIST
+	PREFERENCE_STRING_ARRAY,
+	PREFERENCE_STRING_ARRAY_AS_QUARKS
 } PreferenceType;
 
 /*
@@ -94,7 +94,7 @@ preferences_gconf_value_get_int (const GConfValue *value)
 	if (value == NULL) {
 		return 0;
 	}
-	g_return_val_if_fail (value->type == GCONF_VALUE_INT, 0);
+	g_assert (value->type == GCONF_VALUE_INT);
 	return gconf_value_get_int (value);
 }
 
@@ -104,7 +104,7 @@ preferences_gconf_value_get_bool (const GConfValue *value)
 	if (value == NULL) {
 		return FALSE;
 	}
-	g_return_val_if_fail (value->type == GCONF_VALUE_BOOL, FALSE);
+	g_assert (value->type == GCONF_VALUE_BOOL);
 	return gconf_value_get_bool (value);
 }
 
@@ -114,32 +114,39 @@ preferences_gconf_value_get_string (const GConfValue *value)
 	if (value == NULL) {
 		return g_strdup ("");
 	}
-	g_return_val_if_fail (value->type == GCONF_VALUE_STRING, g_strdup (""));
+	g_assert (value->type == GCONF_VALUE_STRING);
 	return g_strdup (gconf_value_get_string (value));
 }
 
-static EelStringList *
-preferences_gconf_value_get_string_list (const GConfValue *value)
+static char **
+preferences_gconf_value_get_string_array (const GConfValue *value)
 {
-	GSList *slist;
-	EelStringList *result;
+	GSList *slist, *l;
+	GPtrArray *result;
 
 	if (value == NULL) {
-		return eel_string_list_new (TRUE);
+		return NULL;
 	}
-	g_return_val_if_fail (value->type == GCONF_VALUE_LIST, eel_string_list_new (TRUE));
-	g_return_val_if_fail (gconf_value_get_list_type (value) == GCONF_VALUE_STRING, eel_string_list_new (TRUE));
+
+	g_assert (value->type == GCONF_VALUE_LIST);
+	g_assert (gconf_value_get_list_type (value) == GCONF_VALUE_STRING);
 
 	slist = eel_gconf_value_get_string_list (value);
-	result = eel_string_list_new_from_g_slist (slist, TRUE);
-	eel_g_slist_free_deep (slist);
- 	return result;
+
+	result = g_ptr_array_new ();
+	for (l = slist; l != NULL; l = l->next) {
+		g_ptr_array_add (result, l->data);
+	}
+	g_slist_free (slist);
+	g_ptr_array_add (result, NULL);
+
+ 	return (char **) g_ptr_array_free (result, FALSE);
 }
 
 static const char *
 preferences_peek_storage_path (void)
 {
-	g_return_val_if_fail (storage_path != NULL, NULL);
+	g_assert (storage_path != NULL);
 
 	return storage_path;
 }
@@ -147,7 +154,7 @@ preferences_peek_storage_path (void)
 static void
 preferences_set_storage_path (const char *new_storage_path)
 {
-	g_return_if_fail (eel_strlen (new_storage_path) > 0);
+	g_assert (eel_strlen (new_storage_path) > 0);
 
 	/* Make sure the path is indeed different */
 	if (eel_str_is_equal (new_storage_path, storage_path)) {
@@ -180,8 +187,8 @@ preferences_get_value (const char *name)
 	char *key;
 	PreferencesEntry *entry;
 	
-	g_return_val_if_fail (name != NULL, 0);
-	g_return_val_if_fail (preferences_is_initialized (), 0);
+	g_assert (name != NULL);
+	g_assert (preferences_is_initialized ());
 
 	key = preferences_key_make (name);
 	result = eel_gconf_get_value (key);
@@ -202,7 +209,7 @@ preferences_get_value (const char *name)
 static gboolean
 preferences_preference_is_gconf_key (const char *name)
 {
-	g_return_val_if_fail (name != NULL, FALSE);
+	g_assert (name != NULL);
 	
 	if (eel_str_has_prefix (name, "/")) {
 		return FALSE;
@@ -214,7 +221,7 @@ preferences_preference_is_gconf_key (const char *name)
 static char *
 preferences_key_make (const char *name)
 {
-	g_return_val_if_fail (name != NULL, NULL);
+	g_assert (name != NULL);
 	
 	if (!preferences_preference_is_gconf_key (name)) {
 		return g_strdup (name);
@@ -233,7 +240,7 @@ preferences_get_default_value (const char *name)
 	PreferencesEntry *entry;
 	char *key;
 
-	g_return_val_if_fail (name != NULL, NULL);
+	g_assert (name != NULL);
 
 	key = preferences_key_make (name);
 
@@ -250,13 +257,39 @@ preferences_get_default_value (const char *name)
         return result;
 }
 
+static int
+preferences_callback_entry_compare (gconstpointer a,
+				    gconstpointer b)
+{
+	const PreferencesCallbackEntry *a_entry = a;
+	const PreferencesCallbackEntry *b_entry = b;
+
+	if (a_entry->callback < b_entry->callback) {
+		return -1;
+	}
+
+	if (a_entry->callback > b_entry->callback) {
+		return +1;
+	}
+
+	if (a_entry->callback_data < b_entry->callback_data) {
+		return -1;
+	}
+
+	if (a_entry->callback_data > b_entry->callback_data) {
+		return +1;
+	}
+
+	return 0;
+}
+
 /* Public preferences functions */
 
 gboolean
 eel_preferences_get_is_invisible (const char *name)
 {
-	g_return_val_if_fail (name != NULL, FALSE);
-	g_return_val_if_fail (preferences_is_initialized (), FALSE);
+	g_assert (name != NULL);
+	g_assert (preferences_is_initialized ());
 
 	return preferences_global_table_lookup_or_insert (name)->invisible;
 }
@@ -344,7 +377,7 @@ eel_preferences_get_enum (const char *name)
 	int               ret_val;
 	char             *str_value;
 	GConfValue       *value;
-	EelEnumeration   *enumeration;
+	const EelEnumeration   *enumeration;
 	PreferencesEntry *entry;
 
 	g_return_val_if_fail (name != NULL, 0);
@@ -376,8 +409,8 @@ eel_preferences_get_enum (const char *name)
 		return 0;
 	}
 
-	ret_val = eel_enumeration_get_sub_value (enumeration, str_value);
-	
+	ret_val = eel_enumeration_get_value_for_name (enumeration, str_value);
+
 	g_free (str_value);
 
 	return ret_val;
@@ -388,7 +421,7 @@ eel_preferences_set_enum (const char *name,
 			  int         int_value)
 {
 	const char       *str_value;
-	EelEnumeration   *enumeration;
+	const EelEnumeration   *enumeration;
 	PreferencesEntry *entry;
 
 	g_return_if_fail (name != NULL);
@@ -405,7 +438,7 @@ eel_preferences_set_enum (const char *name,
 		return;
 	}
 
-	str_value = eel_enumeration_get_sub_name (enumeration, int_value);
+	str_value = eel_enumeration_get_name_for_value (enumeration, int_value);
 
 	if (str_value == NULL) {
 		g_warning ("No enum match for '%d'", int_value);
@@ -432,6 +465,7 @@ eel_preferences_set (const char *name,
 		eel_gconf_set_string (key, string_value);
 	}
 	g_free (key);
+	g_free (old_value);
 }
 
 char *
@@ -440,8 +474,8 @@ eel_preferences_get (const char *name)
  	char *result;
 	GConfValue *value;
 
-	g_return_val_if_fail (name != NULL, 0);
-	g_return_val_if_fail (preferences_is_initialized (), 0);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (preferences_is_initialized (), NULL);
 
 	value = preferences_get_value (name);
 	result = preferences_gconf_value_get_string (value);
@@ -451,59 +485,74 @@ eel_preferences_get (const char *name)
 }
 
 void
-eel_preferences_set_string_list (const char *name,
-				 const EelStringList *string_list_value)
+eel_preferences_set_string_array (const char  *name,
+				  char       **strv_value)
 {
-	char *key;
 	GSList *slist;
+	int i;
+	char *key;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (preferences_is_initialized ());
 
-	slist = eel_string_list_as_g_slist (string_list_value);
+	slist = NULL;
+	if (strv_value != NULL) {
+		for (i = 0; strv_value[i] != NULL; i++) {
+			slist = g_slist_prepend (slist, strv_value[i]);
+		}
+		slist = g_slist_reverse (slist);
+	}
+
 	key = preferences_key_make (name);
 	eel_gconf_set_string_list (key, slist);
 	g_free (key);
 
-	eel_g_slist_free_deep (slist);
+	g_slist_free (slist);
 }
 
 static gboolean
-string_list_is_valid (const EelStringList *string_list,
-		      const char *enumeration_id)
+string_array_is_valid (char **strv, const char *enumeration_id)
 {
 	guint i;
-	char *nth;
-	gboolean bad;
+	gboolean res;
 
-	g_return_val_if_fail (string_list != NULL, FALSE);
-	g_return_val_if_fail (enumeration_id != NULL, FALSE);
+	g_assert (strv != NULL);
+	g_assert (enumeration_id != NULL);
 
-	bad = FALSE;
-	for (i = 0; i < eel_string_list_get_length (string_list) && !bad; i++) {
-		nth = eel_string_list_nth (string_list, i);
-		bad = !eel_enumeration_id_contains_name (enumeration_id, nth);
-		g_free (nth);
+	res = TRUE;
+	for (i = 0; strv[i] != NULL; i++) {
+		const EelEnumeration *enumeration;
+
+		enumeration = eel_enumeration_lookup (enumeration_id);
+		if (!enumeration) {
+			res = FALSE;
+			break;
+		}
+
+		if (!eel_enumeration_contains_name (enumeration, strv[i])) {
+			res = FALSE;
+			break;
+		}
 	}
 
-	return !bad;
+	return res;
 }
 
-EelStringList *
-eel_preferences_get_string_list (const char *name)
+char **
+eel_preferences_get_string_array (const char *name)
 {
- 	EelStringList *result;
+ 	char **result;
 	GConfValue *value;
 	PreferencesEntry *entry;
 	GConfValue *default_value;
 
-	g_return_val_if_fail (name != NULL, 0);
-	g_return_val_if_fail (preferences_is_initialized (), 0);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (preferences_is_initialized (), NULL);
 
 	value = preferences_get_value (name);
-	result = preferences_gconf_value_get_string_list (value);
+	result = preferences_gconf_value_get_string_array (value);
 	eel_gconf_value_free (value);
-	
+
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
 
@@ -513,22 +562,53 @@ eel_preferences_get_string_list (const char *name)
 	}
 
 	/* Do a sanity check on the validity of the values */
-	if (string_list_is_valid (result, entry->enumeration_id)) {
+	if (string_array_is_valid (result, entry->enumeration_id)) {
 		return result;
 	}
 
 	/* Forget the bad value and use the default instead */
-	eel_string_list_free (result);
+	g_strfreev (result);
 
 	default_value = preferences_get_default_value (name);
 	if (default_value) {		
-		result = preferences_gconf_value_get_string_list (default_value);
+		result = preferences_gconf_value_get_string_array (default_value);
 		gconf_value_free (default_value);
 	}
 
 	return result;
 }
 
+void
+eel_preferences_unset (const char *name)
+{
+	char *key;
+
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (preferences_is_initialized ());
+
+	key = preferences_key_make (name);
+	
+	eel_gconf_unset (key);
+
+	g_free (key);
+}
+
+gboolean
+eel_preferences_key_is_writable (const char *name)
+{
+	gboolean result;
+	char *key;
+	
+	g_return_val_if_fail (name != NULL, 0);
+	g_return_val_if_fail (preferences_is_initialized (), 0);
+
+	key = preferences_key_make (name);
+	result = eel_gconf_key_is_writable (key);
+	g_free (key);
+
+	return result;
+}
+
 /**
  * preferences_callback_entry_invoke_function
  *
@@ -543,7 +623,7 @@ preferences_callback_entry_invoke_function (gpointer data,
 {
 	PreferencesCallbackEntry *callback_entry;
 	
-	g_return_if_fail (data != NULL);
+	g_assert (data != NULL);
 	
 	callback_entry = data;
 
@@ -553,11 +633,7 @@ preferences_callback_entry_invoke_function (gpointer data,
 static void
 preferences_entry_invoke_callbacks (PreferencesEntry *entry)
 {
-	GConfValue *new_value;
-	
-	g_return_if_fail (entry != NULL);
-
-	new_value = preferences_get_value (entry->name);
+	g_assert (entry != NULL);
 
 	/* Update the auto storage preferences */
 	if (entry->auto_storage_list != NULL) {
@@ -578,8 +654,8 @@ update_auto_string (gpointer data, gpointer callback_data)
 	char **storage;
 	const char *value;
 
-	g_return_if_fail (data != NULL);
-	g_return_if_fail (callback_data != NULL);
+	g_assert (data != NULL);
+	g_assert (callback_data != NULL);
 
 	storage = (char **)data;
 	value = (const char *)callback_data;
@@ -589,25 +665,49 @@ update_auto_string (gpointer data, gpointer callback_data)
 }
 
 static void
-update_auto_string_list (gpointer data, gpointer callback_data)
+update_auto_string_array (gpointer data, gpointer callback_data)
 {
-	EelStringList **storage;
-	const EelStringList *value;
+	char ***storage;
+	char **value;
 	
-	g_return_if_fail (data != NULL);
-	g_return_if_fail (callback_data != NULL);
+	g_assert (data != NULL);
+	g_assert (callback_data != NULL);
 
-	storage = (EelStringList **)data;
-	value = (const EelStringList *)callback_data;
+	storage = (char ***)data;
+	value = (char **)callback_data;
+
+	g_strfreev (*storage);
+	*(char ***)storage = value ? g_strdupv (value) : NULL;
+}
+
+static void
+update_auto_string_array_as_quarks (gpointer data, gpointer callback_data)
+{
+	GQuark **storage;
+	char **value;
+	int i = 0;
 	
-	eel_string_list_free (*storage);
-	*(EelStringList **)storage = eel_string_list_copy (value);
+	g_assert (data != NULL);
+	g_assert (callback_data != NULL);
+
+	storage = (GQuark **)data;
+	value = (char **)callback_data;
+
+	g_free (*storage);
+	*storage = g_new (GQuark, g_strv_length (value) + 1);
+
+	if (value != NULL) {
+		for (i = 0; value[i] != NULL; ++i) {
+			(*storage)[i] = g_quark_from_string (value[i]);
+		}
+	}
+	(*storage)[i] = 0;
 }
 
 static void
 update_auto_integer_or_boolean (gpointer data, gpointer callback_data)
 {
-	g_return_if_fail (data != NULL);
+	g_assert (data != NULL);
 
 	*(int *)data = GPOINTER_TO_INT (callback_data);
 }
@@ -616,7 +716,7 @@ static void
 preferences_entry_update_auto_storage (PreferencesEntry *entry)
 {
 	char *new_string_value;
-	EelStringList *new_string_list_value;
+	char **new_string_array_value;
 	int new_int_value;
 	gboolean new_boolean_value;
 
@@ -635,12 +735,19 @@ preferences_entry_update_auto_storage (PreferencesEntry *entry)
 			g_free (new_string_value);
 		}
 		break;
-	case PREFERENCE_STRING_LIST:
-		new_string_list_value = eel_preferences_get_string_list (entry->name);
+	case PREFERENCE_STRING_ARRAY:
+		new_string_array_value = eel_preferences_get_string_array (entry->name);
 		g_list_foreach (entry->auto_storage_list,
-				update_auto_string_list,
-				new_string_list_value);
-		eel_string_list_free (new_string_list_value);
+				update_auto_string_array,
+				new_string_array_value);
+		g_strfreev (new_string_array_value);
+		break;
+	case PREFERENCE_STRING_ARRAY_AS_QUARKS:
+		new_string_array_value = eel_preferences_get_string_array (entry->name);
+		g_list_foreach (entry->auto_storage_list,
+				update_auto_string_array_as_quarks,
+				new_string_array_value);
+		g_strfreev (new_string_array_value);
 		break;
 	case PREFERENCE_INTEGER:
 		new_int_value = eel_preferences_get_integer (entry->name);
@@ -665,9 +772,9 @@ preferences_something_changed_notice (GConfClient *client,
 				      GConfEntry *entry, 
 				      gpointer notice_data)
 {
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->key != NULL);
-	g_return_if_fail (notice_data != NULL);
+	g_assert (entry != NULL);
+	g_assert (entry->key != NULL);
+	g_assert (notice_data != NULL);
 
 	preferences_entry_invoke_callbacks (notice_data);
 }
@@ -686,7 +793,7 @@ preferences_entry_ensure_gconf_connection (PreferencesEntry *entry)
 		return;
 	}
 		
-	g_return_if_fail (entry->name != NULL);
+	g_assert (entry->name != NULL);
 
 	key = preferences_key_make (entry->name);
 
@@ -695,7 +802,7 @@ preferences_entry_ensure_gconf_connection (PreferencesEntry *entry)
 								 entry);
 	g_free (key);
 
-	g_return_if_fail (entry->gconf_connection_id != EEL_GCONF_UNDEFINED_CONNECTION);
+	g_assert (entry->gconf_connection_id != EEL_GCONF_UNDEFINED_CONNECTION);
 }
 
 /**
@@ -713,19 +820,22 @@ preferences_entry_add_callback (PreferencesEntry *entry,
 				gpointer callback_data)
 {
 	PreferencesCallbackEntry *callback_entry;
+	GList *l;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (callback != NULL);
+	g_assert (entry != NULL);
+	g_assert (callback != NULL);
 
 	callback_entry = g_new0 (PreferencesCallbackEntry, 1);
 	callback_entry->callback = callback;
 	callback_entry->callback_data = callback_data;
-	
-	g_return_if_fail (callback_entry != NULL);
-	
-	entry->callback_list = g_list_append (entry->callback_list, callback_entry);
 
-	preferences_entry_ensure_gconf_connection (entry);
+	l = g_list_find_custom (entry->callback_list, callback_entry, preferences_callback_entry_compare);
+	if (l == NULL) {
+		entry->callback_list = g_list_append (entry->callback_list, callback_entry);
+		preferences_entry_ensure_gconf_connection (entry);
+	} else {
+		g_warning ("Trying to add a callback for %s that already exists.", entry->name);
+	}
 }
 
 /**
@@ -743,10 +853,13 @@ preferences_entry_add_auto_storage (PreferencesEntry *entry,
 				    gpointer storage,
 				    PreferenceType type)
 {
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (storage != NULL);
-	g_return_if_fail (entry->type == 0 || entry->type == type);
-	g_return_if_fail (g_list_find (entry->auto_storage_list, storage) == NULL);
+	g_assert (entry != NULL);
+	g_assert (storage != NULL);
+	g_assert (entry->type == 0 || entry->type == type);
+	if (g_list_find (entry->auto_storage_list, storage) != NULL) {
+		g_warning ("Trying to add an auto storage for %s that already exists.", entry->name);
+		return;
+	}
 
 	entry->type = type;
 	
@@ -784,32 +897,25 @@ preferences_entry_remove_callback (PreferencesEntry *entry,
 				   EelPreferencesCallback callback,
 				   gpointer callback_data)
 {
-	GList *new_list;
-	const GList *node;
+	PreferencesCallbackEntry cb_entry;
+	GList *l;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (callback != NULL);
-	g_return_if_fail (entry->callback_list != NULL);
-	
-	new_list = g_list_copy (entry->callback_list);
-	
-	for (node = new_list; node != NULL; node = node->next) {
-		PreferencesCallbackEntry *callback_entry = node->data;
-		
-		g_return_if_fail (callback_entry != NULL);
-		
-		if (callback_entry->callback == callback &&
-		    callback_entry->callback_data == callback_data) {
-			entry->callback_list = g_list_remove (entry->callback_list, 
-							      callback_entry);
-			
-			preferences_callback_entry_free (callback_entry);
-		}
+	g_assert (entry != NULL);
+	g_assert (callback != NULL);
+
+	cb_entry.callback = callback;
+	cb_entry.callback_data = callback_data;
+
+	l = g_list_find_custom (entry->callback_list, &cb_entry, preferences_callback_entry_compare);
+	if (l != NULL) {
+		preferences_callback_entry_free (l->data);
+		entry->callback_list = g_list_delete_link (entry->callback_list, l);
+		preferences_entry_check_remove_connection (entry);
+	} else {
+		g_warning ("Trying to remove a callback for %s without adding it first.", entry->name);
 	}
 
-	g_list_free (new_list);
-
-	preferences_entry_check_remove_connection (entry);
+	g_assert (g_list_find_custom (entry->callback_list, &cb_entry, preferences_callback_entry_compare) == NULL);
 }
 
 /**
@@ -827,16 +933,16 @@ preferences_entry_remove_auto_storage (PreferencesEntry *entry,
 	const GList *node;
 	gpointer storage_in_entry;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (storage != NULL);
-	g_return_if_fail (entry->auto_storage_list != NULL);
+	g_assert (entry != NULL);
+	g_assert (storage != NULL);
+	g_assert (entry->auto_storage_list != NULL);
 	
 	new_list = g_list_copy (entry->auto_storage_list);
 	
 	for (node = new_list; node != NULL; node = node->next) {
 		storage_in_entry = node->data;
 		
-		g_return_if_fail (storage_in_entry != NULL);
+		g_assert (storage_in_entry != NULL);
 		
 		if (storage_in_entry == storage) {
 			entry->auto_storage_list = g_list_remove (entry->auto_storage_list, 
@@ -846,12 +952,15 @@ preferences_entry_remove_auto_storage (PreferencesEntry *entry,
 			case PREFERENCE_STRING:
 				update_auto_string (storage, NULL);
 				break;
-			case PREFERENCE_STRING_LIST:
-				update_auto_string_list (storage, NULL);
+			case PREFERENCE_STRING_ARRAY:
+				update_auto_string_array (storage, NULL);
+				break;
+			case PREFERENCE_STRING_ARRAY_AS_QUARKS:
+				update_auto_string_array_as_quarks (storage, NULL);
 				break;
 			case PREFERENCE_BOOLEAN:
 			case PREFERENCE_INTEGER:
-				update_auto_integer_or_boolean (storage, 0);
+				update_auto_integer_or_boolean (storage, NULL);
 				break;
 			default:
 				g_warning ("unexpected preference type %d in preferences_entry_remove_auto_storage", entry->type);
@@ -873,7 +982,7 @@ preferences_entry_remove_auto_storage (PreferencesEntry *entry,
 static void
 preferences_callback_entry_free (PreferencesCallbackEntry *callback_entry)
 {
-	g_return_if_fail (callback_entry != NULL);
+	g_assert (callback_entry != NULL);
 
 	callback_entry->callback = NULL;
 	callback_entry->callback_data = NULL;
@@ -893,7 +1002,7 @@ static void
 preferences_callback_entry_free_func (gpointer	data,
 				      gpointer	callback_data)
 {
-	g_return_if_fail (data != NULL);
+	g_assert (data != NULL);
 	
 	preferences_callback_entry_free (data);
 }
@@ -907,7 +1016,7 @@ preferences_callback_entry_free_func (gpointer	data,
 static void
 preferences_entry_free (PreferencesEntry *entry)
 {
-	g_return_if_fail (entry != NULL);
+	g_assert (entry != NULL);
 
 	eel_gconf_notification_remove (entry->gconf_connection_id);
 	entry->gconf_connection_id = EEL_GCONF_UNDEFINED_CONNECTION;
@@ -963,6 +1072,12 @@ preferences_global_table_free (void)
 	storage_path = NULL;
 }
 
+static void
+preferences_uninitialize (void)
+{
+	initialized = FALSE;
+}
+
 static GHashTable *
 preferences_global_table_get_global (void)
 {
@@ -974,6 +1089,8 @@ preferences_global_table_get_global (void)
 		if (!at_exit_handler_added) {
 			at_exit_handler_added = TRUE;
 			eel_debug_call_at_shutdown (preferences_global_table_free);
+			/* ensure that we catch calls to preferences functions after eel shutdown */
+			eel_debug_call_at_shutdown (preferences_uninitialize);
 		}
 	}
 	
@@ -983,8 +1100,8 @@ preferences_global_table_get_global (void)
 static PreferencesEntry *
 preferences_global_table_lookup (const char *name)
 {
-	g_return_val_if_fail (name != NULL, NULL);
-	g_return_val_if_fail (preferences_global_table_get_global () != NULL, NULL);
+	g_assert (name != NULL);
+	g_assert (preferences_global_table_get_global () != NULL);
 	
 	return g_hash_table_lookup (preferences_global_table_get_global (), name);
 }
@@ -994,16 +1111,16 @@ preferences_global_table_insert (const char *name)
 {
 	PreferencesEntry *entry;
 
-	g_return_val_if_fail (name != NULL, NULL);
-	g_return_val_if_fail (preferences_global_table_get_global () != NULL, NULL);
-	g_return_val_if_fail (preferences_global_table_lookup (name) == NULL, NULL);
+	g_assert (name != NULL);
+	g_assert (preferences_global_table_get_global () != NULL);
+	g_assert (preferences_global_table_lookup (name) == NULL);
 	
 	entry = g_new0 (PreferencesEntry, 1);
 	entry->name = g_strdup (name);
 
 	g_hash_table_insert (preferences_global_table_get_global (), entry->name, entry);
 
-	g_return_val_if_fail (entry == preferences_global_table_lookup (name), NULL);
+	g_assert (entry == preferences_global_table_lookup (name));
 
 	return entry;
 }
@@ -1013,7 +1130,7 @@ preferences_global_table_lookup_or_insert (const char *name)
 {
 	PreferencesEntry *entry;
 
-	g_return_val_if_fail (name != NULL, NULL);
+	g_assert (name != NULL);
 	
 	entry = preferences_global_table_lookup (name);
 
@@ -1066,11 +1183,11 @@ eel_preferences_add_auto_string (const char *name,
 }
 
 void
-eel_preferences_add_auto_string_list (const char *name,
-				      const EelStringList **storage)
+eel_preferences_add_auto_string_array (const char   *name,
+				       char       ***storage)
 {
 	PreferencesEntry *entry;
-	EelStringList *value;
+	char **value;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (storage != NULL);
@@ -1079,11 +1196,32 @@ eel_preferences_add_auto_string_list (const char *name,
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
 
-	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_LIST);
+	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_ARRAY);
 
-	value = eel_preferences_get_string_list (entry->name);
-	update_auto_string_list (storage, value);
-	eel_string_list_free (value);
+	value = eel_preferences_get_string_array (entry->name);
+	update_auto_string_array (storage, value);
+	g_strfreev (value);
+}
+
+void
+eel_preferences_add_auto_string_array_as_quarks (const char *name,
+						 GQuark **storage)
+{
+	PreferencesEntry *entry;
+	char **value;
+
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (storage != NULL);
+	g_return_if_fail (preferences_is_initialized ());
+
+	entry = preferences_global_table_lookup_or_insert (name);
+	g_assert (entry != NULL);
+
+	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING_ARRAY_AS_QUARKS);
+
+	value = eel_preferences_get_string_array (entry->name);
+	update_auto_string_array_as_quarks (storage, value);
+	g_free (value);
 }
 
 void
@@ -1168,8 +1306,8 @@ eel_preferences_remove_auto_string (const char *name,
 }
 
 void
-eel_preferences_remove_auto_string_list (const char *name,
-					 const EelStringList **storage)
+eel_preferences_remove_auto_string_array (const char *name,
+					  char ***storage)
 {
 	PreferencesEntry *entry;
 
@@ -1240,9 +1378,13 @@ preferences_while_alive_disconnector (gpointer callback_data, GObject *where_obj
 
 	data = callback_data;
 
-	eel_preferences_remove_callback (data->name,
-					 data->callback,
-					 data->callback_data);
+	/* we might have survived an eel shutdown, which
+	 * already cleared all the callbacks */
+	if (preferences_is_initialized ()) {
+		eel_preferences_remove_callback (data->name,
+						 data->callback,
+						 data->callback_data);
+	}
 
 	g_free (data->name);
 	g_free (data);
@@ -1360,8 +1502,8 @@ preferences_set_emergency_fallback_stealing_value (const char *name,
 {
 	PreferencesEntry *entry;
 
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (preferences_is_initialized ());
+	g_assert (name != NULL);
+	g_assert (preferences_is_initialized ());
 
 	entry = preferences_global_table_lookup_or_insert (name);
 	g_assert (entry != NULL);
@@ -1417,34 +1559,32 @@ eel_preferences_set_emergency_fallback_boolean (const char *name,
 	preferences_set_emergency_fallback_stealing_value (name, gconf_value);
 }
 
-static void
-listify_strings_foreach (const char *string,
-			 gpointer callback_data)
-{
-	GSList **listp = callback_data;
-	GConfValue *value;
-
-	value = gconf_value_new (GCONF_VALUE_STRING);
-	gconf_value_set_string (value, string);
-	
-	*listp = g_slist_prepend (*listp, value);
-}
 
 void
-eel_preferences_set_emergency_fallback_string_list (const char    *name,
-						    EelStringList *value)
+eel_preferences_set_emergency_fallback_string_array (const char  *name,
+						     char       **value)
 {
 	GConfValue *gconf_value;
 	GSList *list;
+	int i;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (value != NULL);
-	
-	gconf_value = gconf_value_new (GCONF_VALUE_LIST);
 
+	gconf_value = gconf_value_new (GCONF_VALUE_LIST);
 	gconf_value_set_list_type (gconf_value, GCONF_VALUE_STRING);
+
 	list = NULL;
-	eel_string_list_for_each (value, listify_strings_foreach, &list);
+	for (i = 0; value[i] != NULL; ++i)
+	{
+		GConfValue *v;
+
+		v = gconf_value_new (GCONF_VALUE_STRING);
+		gconf_value_set_string (v, value[i]);
+
+		list = g_slist_prepend (list, v);
+	}
+
 	gconf_value_set_list_nocopy (gconf_value, g_slist_reverse (list));
 
 	preferences_set_emergency_fallback_stealing_value (name, gconf_value);
